#include <iostream>
#include <string>
#include "DVector.hh"
#include "Time.hh"
#include "Interval.hh"
#include "TSeries.hh"
#include "constant.hh"
#include "math.h"

#include "SlopeFilter.hh"

using namespace std;

SlopeFilter::
SlopeFilter(void)
{
  _order = 0;
  _in_use = false;
}

SlopeFilter::
SlopeFilter(unsigned order, TSeries tsinit)
{
  // set name
  _channel_name = tsinit.getName() ;
  // initialize order and sampling rate
  init(order,1/((tsinit.getTStep()).GetSecs()));
}

SlopeFilter::
SlopeFilter(unsigned order,double samprate)
{
  // set a dummy channel name
  _channel_name = "unknown";
  // initialize order and sampling rate
  init(order,samprate);
}

SlopeFilter::
SlopeFilter(const SlopeFilter& y) {
  _in_use = y._in_use;
  if( _in_use ) {
    _channel_name = y._channel_name;
    _sample_rate = y._sample_rate;
    _order = y._order;
    history_buffer = new double ( *(y.history_buffer) );
    _buffer_counter = y._buffer_counter;
    _buffer_length = y._buffer_length;
  }
}    

void 
SlopeFilter::init(unsigned order, double samprate)
{
  // set order
  // cout << "setting order" << endl;
  _order = order;
  // set sampling rate
  _sample_rate = samprate;
  // allocate history buffer space
  _buffer_counter = 0;
  _buffer_length = (int)_order - 1;
  history_buffer = new double [_buffer_length];
  // set filter in use
  _in_use = true;
}
  
SlopeFilter::
~SlopeFilter(void)
{
  if( _in_use ) {
    delete [] history_buffer;
  }
}

void
SlopeFilter::setFilterParams(unsigned order, TSeries tsinit) {
  // set order
  _order = order;
  // set name
  _channel_name = tsinit.getName() ;
  // get sampling rate
  _sample_rate = 1/((tsinit.getTStep()).GetSecs());
  // allocate history buffer space
  _buffer_counter = 0;
  _buffer_length = (int)_order - 1;
  history_buffer = new double [_buffer_length];
  // set filter in use bit
  _in_use = true;
  return;
}

TSeries
SlopeFilter::apply(const TSeries& data_in)
{
  if( !_in_use ) {
    return data_in;
  }
  // create a copy of the data converted to data type double
  TSeries databuffer(data_in);
  databuffer.Convert(DVector::t_double);
  int datalength = databuffer.getNSample();  

  // allocate memory for a new array of doubles for the filtered data
  double* pdata = new double [ datalength ];  
  double* dataout = new double [ datalength ];
  // get the input data to the array
  databuffer.getData(data_in.getNSample(), pdata);

  // filter the data
  double dataelement;
  double sumof_ts = 0;
  double sumof_t = 0;
  double sumof_s = 0;
  double sumof_tt = 0;
  for(int i=0; i < datalength ; ++i) {
    sumof_ts = 0;
    sumof_t = 0;
    sumof_s = 0;
    sumof_tt = 0;
    double dc;
    for( int j=0 ; j<_buffer_length; ++j ) {
      if( (i+j) < _buffer_length ) {
	dataelement = history_buffer[i+j];
      } else {
	dataelement = pdata[i+j-_buffer_length];
      }
      dc = (double)j;
      sumof_ts += dataelement * dc;
      sumof_s += dataelement;
      sumof_t += dc;
      sumof_tt += (dc*dc);
    }
    // modify history buffer
    if( i < _buffer_length ) {
      history_buffer[i] = pdata[ datalength - _buffer_length + i ];
    }
    // calculate slope statistic
    dataout[i] = (sumof_ts - sumof_t*sumof_s) / (sumof_tt - pow(sumof_t,2) );

    // cout << i << " : " << pdata[i] << "," << dataout[i] << endl;

  }

  databuffer.setData( databuffer.getStartTime(),
		      databuffer.getTStep(),
		      dataout, 
		      databuffer.getNSample() );
  delete [] pdata;
  delete [] dataout;
  return databuffer;
}

void
SlopeFilter::
dataCheck(const TSeries& data_in) const {
  cout << "not implemented" << endl;
  return;
}

void 
SlopeFilter::
reset(void)
{
  if( _in_use ) {
    for(int i=0;i<_buffer_length;++i) {
      history_buffer[i] = 0;
    }
  }
  return;
}

bool
SlopeFilter::
inUse(void) const {
  return _in_use;
}

Time
SlopeFilter::
getStartTime(void) const {
  cout << "not implemented" << endl;
  Time current=Now();
  return current;
}

Time
SlopeFilter::
getCurrentTime(void) const {
  cout << "not implemented" << endl;
  Time current=Now();
  return current;
}

